@@ -7,6 +7,12 @@ import sys, os
 import locale
 import unittest
 
+# assertRaisesRegexp and assertRegexpMatches have been renamed in
+# unittest for python 3, but not in python 2 (at least yet).
+if hasattr(unittest.TestCase, 'assertRaisesRegex'):
+    unittest.TestCase.assertRaisesRegexp = unittest.TestCase.assertRaisesRegex
+    unittest.TestCase.assertRegexpMatches = unittest.TestCase.assertRegex
+
 import lg_testutils # Found in the same directory of this test script
 
 # Show information on this program run
@@ -58,8 +64,15 @@ class AAALinkTestCase(unittest.TestCase):
 
 class AADictionaryTestCase(unittest.TestCase):
     def test_open_nonexistent_dictionary(self):
-        self.assertRaises(LG_DictionaryError, Dictionary, 'No such language test 1')
-        self.assertRaises(LG_Error, Dictionary, 'No such language test 2')
+        dummy_lang = "No such language test "
+
+        save_stderr = divert_start(2)
+        self.assertRaises(LG_DictionaryError, Dictionary, dummy_lang + '1')
+        self.assertIn(dummy_lang + '1', save_stderr.divert_end())
+
+        save_stderr = divert_start(2)
+        self.assertRaises(LG_Error, Dictionary, dummy_lang + '2')
+        self.assertIn(dummy_lang + '2', save_stderr.divert_end())
 
 class BParseOptionsTestCase(unittest.TestCase):
     def test_setting_verbosity(self):
@@ -401,13 +414,16 @@ class EErrorFacilityTestCase(unittest.TestCase):
         #LG_Error.set_handler(self.__class__.handler["default"], "bad param")
         #with self.assertRaises(TypeError):
         #    try:
-        #        Dictionary("a visible dummy dict name (bad param test)")
+        #        Dictionary("a dummy dict name (bad param test)")
         #    except LG_Error:
         #        pass
 
         # So test it directly.
-        self.assertRaises(LG_Error, Dictionary, "a visible dummy dict name (bad param test)")
+
+        dummy_lang = "a dummy dict name (bad param test)"
+        self.assertRaises(LG_Error, Dictionary, dummy_lang)
         LG_Error.printall(self.error_handler_test, self) # grab a valid errinfo
+        #self.assertIn(dummy_lang, save_stderr.divert_end())
         self.assertRaisesRegexp(TypeError, "must be an integer",
                                 self.__class__.handler["default"],
                                 self.errinfo, "bad param")
@@ -416,9 +432,12 @@ class EErrorFacilityTestCase(unittest.TestCase):
                                 self.errinfo, clg.lg_None+1)
         self.assertRaises(ValueError, self.__class__.handler["default"],
                           self.errinfo, -1)
+
         try:
             self.param_ok = False
+            save_stdout  = divert_start(1) # Note: Handler parameter is stdout
             self.__class__.handler["default"](self.errinfo, 1)
+            self.assertIn(dummy_lang, save_stdout.divert_end())
             self.param_ok = True
         except (TypeError, ValueError):
             self.assertTrue(self.param_ok)
@@ -492,8 +511,12 @@ class EErrorFacilityTestCase(unittest.TestCase):
         for _ in range(0, 1+self.testleaks):
             self.__class__.handler["previous"] = LG_Error.set_handler(self.__class__.handler["default"])
         self.assertEqual(self.__class__.handler["previous"].__name__, "_default_handler")
+
         self.errinfo = "dummy"
-        self.assertRaises(LG_Error, Dictionary, "a visible dummy dict name (default handler test)")
+        dummy_lang = "a dummy dict name (default handler test)"
+        save_stderr = divert_start(2)
+        self.assertRaises(LG_Error, Dictionary, dummy_lang)
+        self.assertIn(dummy_lang, save_stderr.divert_end())
         self.assertEqual(self.errinfo, "dummy")
 
 class FSATsolverTestCase(unittest.TestCase):
@@ -535,7 +558,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
         self.assertEqual(linkage.link(1),
                          Link(linkage, 1, 'LEFT-WALL','hWV','dWV','is.v'))
         self.assertEqual(linkage.link(2),
-                         Link(linkage, 2, 'LEFT-WALL','Wd','Wd','this.p'))
+                         Link(linkage, 2, 'LEFT-WALL','hWd','Wd','this.p'))
         self.assertEqual(linkage.link(3),
                          Link(linkage, 3, 'this.p','Ss*b','Ss','is.v'))
         self.assertEqual(linkage.link(4),
@@ -664,18 +687,18 @@ class HEnglishLinkageTestCase(unittest.TestCase):
 "\n    +---------------------------->WV---------------------------->+                  |"
 "\n    |           +-----------------------Sp-----------------------+                  |"
 "\n    |           |                  +------------VJlpi------------+                  |"
-"\n    +-----Wd----+          +---E---+---I---+----Op----+          +VJrpi+---Op--+    |"
+"\n    +---->Wd----+          +---E---+---I---+----Op----+          +VJrpi+---Op--+    |"
 "\n    |           |          |       |       |          |          |     |       |    |"
-"\nLEFT-WALL scientists.n sometimes may.v repeat.v experiments.n or.j-v use.v groups.n . "
+"\nLEFT-WALL scientists.n sometimes may.v repeat.v experiments.n or.j-v use.v groups.n ."
 "\n\n")
         sent = 'I enjoy eating bass.'
         linkage = self.parse_sent(sent)[0]
         self.assertEqual(linkage.diagram(),
 "\n    +-----------------Xp----------------+"
 "\n    +---->WV---->+                      |"
-"\n    +--Wd--+-Sp*i+---Pg---+---Ou---+    |"
+"\n    +->Wd--+-Sp*i+---Pg---+---Ou---+    |"
 "\n    |      |     |        |        |    |"
-"\nLEFT-WALL I.p enjoy.v eating.v bass.n-u . "
+"\nLEFT-WALL I.p enjoy.v eating.v bass.n-u ."
 "\n\n")
 
 
@@ -683,9 +706,9 @@ class HEnglishLinkageTestCase(unittest.TestCase):
         linkage = self.parse_sent(sent)[0]
         self.assertEqual(linkage.diagram(),
 "\n    +--->WV--->+     +---------Js--------+"
-"\n    +--Wd--+Spx+--Pp-+   +--DD--+---GN---+"
+"\n    +->Wd--+Spx+--Pp-+   +--DD--+---GN---+"
 "\n    |      |   |     |   |      |        |"
-"\nLEFT-WALL we are.v from the planet.n Gorpon[!] "
+"\nLEFT-WALL we are.v from the planet.n Gorpon[!]"
 "\n\n")
 
 
@@ -858,16 +881,26 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
     testfile = clg.test_data_srcdir + "parses-" + desc + clg.dictionary_get_lang(lgdict._obj) + ".txt"
     parses = open(testfile, "rb")
     diagram = None
+    constituents = None
     sent = None
     lineno = 0
-    opcode_detected = 0 # function sanity check
+
+    # Function code and file format sanity check
+    self.opcode_detected = 0
+    def validate_opcode(ctxt=self, O=False, C=False):
+        ctxt.opcode_detected += 1
+        if O:
+            self.assertFalse(diagram, "at {}:{}: Unfinished diagram entry".format(testfile, lineno))
+        if C:
+            self.assertFalse(constituents, "at {}:{}: Unfinished constituents entry".format(testfile, lineno))
+
     for line in parses:
         lineno += 1
         if sys.version_info > (3, 0):
             line = line.decode('utf-8')
         # Lines starting with I are the input sentences
         if 'I' == line[0]:
-            opcode_detected += 1
+            validate_opcode(O=True, C=True)
             sent = line[1:]
             diagram = ""
             constituents = ""
@@ -877,7 +910,7 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
 
         # Generate the next linkage of the last input sentence
         if 'N' == line[0]:
-            opcode_detected += 1
+            validate_opcode(O=True, C=True)
             diagram = ""
             constituents = ""
             linkage = next(linkages, None)
@@ -886,27 +919,59 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
         # Lines starting with O are the parse diagram
         # It ends with an empty line
         if 'O' == line[0]:
-            opcode_detected += 1
+            validate_opcode(C=True)
             diagram += line[1:]
             if '\n' == line[1] and 1 < len(diagram):
                 self.assertEqual(linkage.diagram(), diagram, "at {}:{}".format(testfile, lineno))
+                diagram = None
 
         # Lines starting with C are the constituent output (type 1)
         # It ends with an empty line
         if 'C' == line[0]:
-            opcode_detected += 1
+            validate_opcode(O=True)
             if '\n' == line[1] and 1 < len(constituents):
                 self.assertEqual(linkage.constituent_tree(), constituents, "at {}:{}".format(testfile, lineno))
-            constituents += line[1:]
+                constituents = None
+            else:
+                constituents += line[1:]
     parses.close()
 
-    self.assertGreaterEqual(opcode_detected, 2, "Nothing has been done for " + testfile)
+    validate_opcode(O=True, C=True)
+    self.assertGreaterEqual(self.opcode_detected, 2, "Nothing has been done for " + testfile)
 
 def warning(*msg):
     progname = os.path.basename(sys.argv[0])
     print("{}: Warning:".format(progname), *msg, file=sys.stderr)
 
 
+import tempfile
+
+class divert_start(object):
+    """ Output diversion. """
+    def __init__(self, fd):
+        """ Divert a file descriptor.
+        The created object is used for restoring the original file descriptor.
+        """
+        self.fd = fd
+        self.savedfd = os.dup(fd)
+        (newfd, self.filename) = tempfile.mkstemp(text=False)
+        os.dup2(newfd, fd)
+        os.close(newfd)
+
+    def divert_end(self):
+        """ Restore a previous diversion and return its content. """
+        if not self.filename:
+            return ""
+        os.lseek(self.fd, os.SEEK_SET, 0)
+        content = os.read(self.fd, 1024) # 1024 is more than needed
+        os.dup2(self.savedfd, self.fd)
+        os.close(self.savedfd)
+        os.unlink(self.filename)
+        self.filename = None
+        return str(content)
+
+    __del__ = divert_end
+
 # Decorate Sentence.parse with eqcost_soretd_parse.
 lg_testutils.add_eqcost_linkage_order(Sentence)
 
