@@ -80,9 +80,17 @@ char * join_path(const char * prefix, const char * suffix)
  */
 static char * custom_data_dir = NULL;
 
+static void free_custom_data_dir(void)
+{
+	free(custom_data_dir);
+}
+
 void dictionary_set_data_dir(const char * path)
 {
-	if (custom_data_dir) free (custom_data_dir);
+	if (custom_data_dir)
+		free(custom_data_dir);
+	else
+		atexit(free_custom_data_dir);
 	custom_data_dir = safe_strdup(path);
 }
 
@@ -193,10 +201,7 @@ void * object_open(const char *filename,
 
 	/* Look for absolute filename.
 	 * Unix: starts with leading slash.
-	 * Windows: starts with C:\  except that the drive letter may differ.
-	 * Note that only native windows C library uses backslashes; mingw
-	 * seems to use forward-slash, from what I can tell.
-	 */
+	 * Windows: starts with C:\  except that the drive letter may differ. */
 	if ((filename[0] == '/')
 #ifdef _WIN32
 		|| ((filename[1] == ':')
