@@ -12,6 +12,7 @@
 /*************************************************************************/
 
 #include <limits.h>
+
 #include "link-includes.h"
 #include "api-structures.h"
 #include "connectors.h"
@@ -35,12 +36,13 @@ struct Table_connector_s
 
 struct count_context_s
 {
+	fast_matcher_t *mchxt;
 	Word *  local_sent;
 	/* int     null_block; */ /* not used, always 1 */
 	bool    islands_ok;
 	bool    null_links;
 	bool    exhausted;
-	int     checktimer;  /* Avoid excess system calls */
+	unsigned checktimer;  /* Avoid excess system calls */
 	int     table_size;
 	int     log2_table_size;
 	Table_connector ** table;
@@ -132,11 +134,11 @@ find_table_pointer(count_context_t *ctxt,
 	/* Create a new connector only if resources are exhausted.
 	 * (???) Huh? I guess we're in panic parse mode in that case.
 	 * checktimer is a device to avoid a gazillion system calls
-	 * to get the timer value. On circa-2009 machines, it results
-	 * in maybe 5-10 timer calls per second.
+	 * to get the timer value. On circa-2017 machines, it results
+	 * in about 0.5-1 timer calls per second.
 	 */
 	ctxt->checktimer ++;
-	if (ctxt->exhausted || ((0 == ctxt->checktimer%1450100) &&
+	if (ctxt->exhausted || ((0 == ctxt->checktimer%(1<<21)) &&
 	                       (ctxt->current_resources != NULL) &&
 	                       resources_exhausted(ctxt->current_resources)))
 	{
@@ -158,6 +160,13 @@ Count_bin* table_lookup(count_context_t * ctxt,
 	if (t == NULL) return NULL; else return &t->count;
 }
 
+#define NO_COUNT -1
+#ifdef PERFORM_COUNT_HISTOGRAMMING
+Count_bin count_unknown = {.total = NO_COUNT};
+#else
+Count_bin count_unknown = NO_COUNT;
+#endif
+
 /**
  * psuedocount is used to check to see if a parse is even possible,
  * so that we don't waste cpu time performing an actual count, only
@@ -170,14 +179,13 @@ Count_bin* table_lookup(count_context_t * ctxt,
  * in the hash table, and its zero, then we know, for sure, that the
  * count is zero.
  */
-static bool pseudocount(count_context_t * ctxt,
+static Count_bin pseudocount(count_context_t * ctxt,
                        int lw, int rw, Connector *le, Connector *re,
                        unsigned int null_count)
 {
 	Count_bin * count = table_lookup(ctxt, lw, rw, le, re, null_count);
-	if (NULL == count) return true;
-	if (hist_total(count) == 0) return false;
-	return true;
+	if (NULL == count) return count_unknown;
+	return *count;
 }
 
 /**
@@ -193,39 +201,43 @@ static int num_optional_words(count_context_t *ctxt, int w1, int w2)
 	return n;
 }
 
-//#define DO_COUNT_TRACE
+#ifdef DEBUG
+#define DO_COUNT_TRACE
+#endif
 
 #ifdef DO_COUNT_TRACE
 #define V(c) (!c?"(nil)":c->string)
-static Count_bin do_count1(int lineno, fast_matcher_t *mchxt,
-                          count_context_t *ctxt,
+static Count_bin do_count1(int lineno, count_context_t *ctxt,
                           int lw, int rw,
                           Connector *le, Connector *re,
                           int null_count);
 
-static Count_bin do_count(int lineno, fast_matcher_t *mchxt,
-                          count_context_t *ctxt,
+static Count_bin do_count(int lineno, count_context_t *ctxt,
                           int lw, int rw,
                           Connector *le, Connector *re,
                           int null_count)
 {
 	static int level;
 
-	level++;
-	printf("%*sdo_count:%d lw=%d rw=%d le=%s re=%s null_count=%d\n",
-		    level*2, "", lineno, lw, rw, V(le), V(re), null_count);
+	if (!verbosity_level(8))
+		return do_count1(lineno, ctxt, lw, rw, le, re, null_count);
+
 	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
-	Count_bin r = do_count1(lineno, mchxt, ctxt, lw, rw, le, re, null_count);
-	printf("%*sreturn%.*s:%d=%lld\n", level*2, "", (!!t)*3, "(M)", lineno, r);
+
+	level++;
+	prt_error("%*sdo_count%.*s:%d lw=%d rw=%d le=%s re=%s null_count=%d\n\\",
+		level*2, "", (!t)*3, "(R)", lineno, lw, rw, V(le), V(re), null_count);
+	Count_bin r = do_count1(lineno, ctxt, lw, rw, le, re, null_count);
+	prt_error("%*sreturn%.*s:%d=%lld\n", level*2, "", (!!t)*3, "(M)", lineno, r);
 	level--;
 
 	return r;
 }
 
-static Count_bin do_count1(int lineno, fast_matcher_t *mchxt,
+static Count_bin do_count1(int lineno,
 #define do_count(...) do_count(__LINE__, __VA_ARGS__)
 #else
-static Count_bin do_count(fast_matcher_t *mchxt,
+static Count_bin do_count(
 #endif
                           count_context_t *ctxt,
                           int lw, int rw,
@@ -313,11 +325,11 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 				if (d->left == NULL)
 				{
 					hist_accumv(&t->count, d->cost,
-						do_count(mchxt, ctxt, w, rw, d->right, NULL, null_count-1));
+						do_count(ctxt, w, rw, d->right, NULL, null_count-1));
 				}
 			}
 			hist_accumv(&t->count, 0.0,
-				do_count(mchxt, ctxt, w, rw, NULL, NULL, null_count-1));
+				do_count(ctxt, w, rw, NULL, NULL, null_count-1));
 		}
 		return t->count;
 	}
@@ -341,18 +353,18 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 	}
 
 	total = zero;
+	fast_matcher_t *mchxt = ctxt->mchxt;
 
 	for (w = start_word; w < end_word; w++)
 	{
-		size_t mlb, mle;
-		mle = mlb = form_match_list(mchxt, w, le, lw, re, rw);
+		size_t mlb = form_match_list(mchxt, w, le, lw, re, rw);
 #ifdef VERIFY_MATCH_LIST
 		int id = get_match_list_element(mchxt, mlb) ?
 		            get_match_list_element(mchxt, mlb)->match_id : 0;
 #endif
-		for (; get_match_list_element(mchxt, mle) != NULL; mle++)
+		for (size_t mle = mlb; get_match_list_element(mchxt, mle) != NULL; mle++)
 		{
-			unsigned int lnull_cnt, rnull_cnt;
+			int lnull_cnt, rnull_cnt;
 			Disjunct *d = get_match_list_element(mchxt, mle);
 			bool Lmatch = d->match_left;
 			bool Rmatch = d->match_right;
@@ -360,14 +372,22 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 #ifdef VERIFY_MATCH_LIST
 			assert(id == d->match_id, "Modified id (%d!=%d)", id, d->match_id);
 #endif
-			/* _p1 avoids a gcc warning about unsafe loop opt */
-			unsigned int null_count_p1 = null_count + 1;
 
-			for (lnull_cnt = 0; lnull_cnt < null_count_p1; lnull_cnt++)
+			for (lnull_cnt = 0; lnull_cnt <= null_count; lnull_cnt++)
 			{
 				bool leftpcount = false;
 				bool rightpcount = false;
-				bool pseudototal = false;
+
+				Count_bin l_any;
+				Count_bin l_cmulti = NO_COUNT;
+				Count_bin l_dmulti = NO_COUNT;
+				Count_bin l_dcmulti = NO_COUNT;
+				Count_bin l_bnr = NO_COUNT;
+				Count_bin r_any = NO_COUNT; /* gcc: may be used uninitialized */
+				Count_bin r_cmulti = NO_COUNT;
+				Count_bin r_dmulti = NO_COUNT;
+				Count_bin r_dcmulti = NO_COUNT;
+				Count_bin r_bnl = NO_COUNT;
 
 				rnull_cnt = null_count - lnull_cnt;
 				/* Now lnull_cnt and rnull_cnt are the costs we're assigning
@@ -376,115 +396,154 @@ static Count_bin do_count(fast_matcher_t *mchxt,
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
 
-				/* First, perform pseudocounting as an optimization. If
+				/* The result count is a sum of multiplications of
+				 * LHS and RHS counts. If one of them is zero, we can skip
+				 * calculating the other one.
+				 *
+				 * So, first perform pseudocounting as an optimization. If
 				 * the pseudocount is zero, then we know that the true
-				 * count will be zero, and so skip counting entirely,
-				 * in that case.
-				 */
+				 * count will be zero.
+				 *
+				 * Cache the result in the l_* and r_* variables, so a table
+				 * lookup can be skipped in cases we cannot skip the actual
+				 * calculation and a table entry exists. */
 				if (Lmatch)
 				{
-					leftpcount = pseudocount(ctxt, lw, w, le->next, d->left->next, lnull_cnt);
+					l_any = pseudocount(ctxt, lw, w, le->next, d->left->next, lnull_cnt);
+					leftpcount = (hist_total(&l_any) != 0);
 					if (!leftpcount && le->multi)
-						leftpcount =
+					{
+						l_cmulti =
 							pseudocount(ctxt, lw, w, le, d->left->next, lnull_cnt);
+						leftpcount |= (hist_total(&l_cmulti) != 0);
+					}
 					if (!leftpcount && d->left->multi)
-						leftpcount =
+					{
+						l_dmulti =
 							pseudocount(ctxt, lw, w, le->next, d->left, lnull_cnt);
+						leftpcount |= (hist_total(&l_dmulti) != 0);
+					}
 					if (!leftpcount && le->multi && d->left->multi)
-						leftpcount =
+					{
+						l_dcmulti =
 							pseudocount(ctxt, lw, w, le, d->left, lnull_cnt);
+						leftpcount |= (hist_total(&l_dcmulti) != 0);
+					}
 				}
 
-				if (Rmatch)
+				if (Rmatch && (leftpcount || (le == NULL)))
 				{
-					rightpcount = pseudocount(ctxt, w, rw, d->right->next, re->next, rnull_cnt);
-					if (!rightpcount && d->right->multi)
-						rightpcount =
-							pseudocount(ctxt, w,rw, d->right, re->next, rnull_cnt);
+					r_any = pseudocount(ctxt, w, rw, d->right->next, re->next, rnull_cnt);
+					rightpcount = (hist_total(&r_any) != 0);
 					if (!rightpcount && re->multi)
-						rightpcount =
+					{
+						r_cmulti =
 							pseudocount(ctxt, w, rw, d->right->next, re, rnull_cnt);
+						rightpcount |= (hist_total(&r_cmulti) != 0);
+					}
+					if (!rightpcount && d->right->multi)
+					{
+						r_dmulti =
+							pseudocount(ctxt, w,rw, d->right, re->next, rnull_cnt);
+						rightpcount |= (hist_total(&r_dmulti) != 0);
+					}
 					if (!rightpcount && d->right->multi && re->multi)
-						rightpcount =
+					{
+						r_dcmulti =
 							pseudocount(ctxt, w, rw, d->right, re, rnull_cnt);
+						rightpcount |= (hist_total(&r_dcmulti) != 0);
+					}
 				}
 
-				/* Total number where links are used on both sides */
-				pseudototal = leftpcount && rightpcount;
+				if (!leftpcount && !rightpcount) continue;
 
-				if (!pseudototal && leftpcount) {
-					/* Evaluate using the left match, but not the right. */
-					pseudototal =
-						pseudocount(ctxt, w, rw, d->right, re, rnull_cnt);
-				}
-				if (!pseudototal && (le == NULL) && rightpcount) {
-					/* Evaluate using the right match, but not the left. */
-					pseudototal =
-						pseudocount(ctxt, lw, w, le, d->left, lnull_cnt);
-				}
-
-				/* If pseudototal is zero (false), that implies that
-				 * we know that the true total is zero. So we don't
-				 * bother counting at all, in that case. */
-				if (pseudototal)
+				if (!(leftpcount && rightpcount))
 				{
-					Count_bin leftcount = zero;
-					Count_bin rightcount = zero;
-					if (Lmatch) {
-						leftcount = do_count(mchxt, ctxt, lw, w, le->next, d->left->next, lnull_cnt);
-						if (le->multi)
-							hist_accumv(&leftcount, d->cost,
-								do_count(mchxt, ctxt, lw, w, le, d->left->next, lnull_cnt));
-						if (d->left->multi)
-							hist_accumv(&leftcount, d->cost,
-								 do_count(mchxt, ctxt, lw, w, le->next, d->left, lnull_cnt));
-						if (le->multi && d->left->multi)
-							hist_accumv(&leftcount, d->cost,
-								do_count(mchxt, ctxt, lw, w, le, d->left, lnull_cnt));
+					if (leftpcount)
+					{
+						/* Evaluate using the left match, but not the right. */
+						l_bnr = do_count(ctxt, w, rw, d->right, re, rnull_cnt);
 					}
-
-					if (Rmatch) {
-						rightcount = do_count(mchxt, ctxt, w, rw, d->right->next, re->next, rnull_cnt);
-						if (d->right->multi)
-							hist_accumv(&rightcount, d->cost,
-								do_count(mchxt, ctxt, w, rw, d->right,re->next, rnull_cnt));
-						if (re->multi)
-							hist_accumv(&rightcount, d->cost,
-								do_count(mchxt, ctxt, w, rw, d->right->next, re, rnull_cnt));
-						if (d->right->multi && re->multi)
-							hist_accumv(&rightcount, d->cost,
-								do_count(mchxt, ctxt, w, rw, d->right, re, rnull_cnt));
+					else if (le == NULL)
+					{
+						/* Evaluate using the right match, but not the left. */
+						r_bnl = do_count(ctxt, lw, w, le, d->left, lnull_cnt);
 					}
+				}
 
-					/* Total number where links are used on both sides */
-					hist_muladd(&total, &leftcount, 0.0, &rightcount);
+#define CACHE_COUNT(c, how_to_count, do_count) \
+{ \
+	Count_bin count = (hist_total(&c) == NO_COUNT) ? do_count : hist_total(&c); \
+	how_to_count; \
+}
+			 /* If the pseudocounting above indicates one of the terms
+			 * in the count multiplication is zero,
+			 * we know that the true total is zero. So we don't
+			 * bother counting the other term at all, in that case. */
+				Count_bin leftcount = zero;
+				Count_bin rightcount = zero;
+				if (leftpcount &&
+				    (rightpcount || (0 != hist_total(&l_bnr))))
+				{
+					CACHE_COUNT(l_any, leftcount = count,
+						do_count(ctxt, lw, w, le->next, d->left->next, lnull_cnt));
+					if (le->multi)
+						CACHE_COUNT(l_cmulti, hist_accumv(&leftcount, d->cost, count),
+							do_count(ctxt, lw, w, le, d->left->next, lnull_cnt));
+					if (d->left->multi)
+						CACHE_COUNT(l_dmulti, hist_accumv(&leftcount, d->cost, count),
+							do_count(ctxt, lw, w, le->next, d->left, lnull_cnt));
+					if (d->left->multi && le->multi)
+						CACHE_COUNT(l_dcmulti, hist_accumv(&leftcount, d->cost, count),
+							do_count(ctxt, lw, w, le, d->left, lnull_cnt));
 
 					if (0 < hist_total(&leftcount))
 					{
 						/* Evaluate using the left match, but not the right */
-						hist_muladdv(&total, &leftcount, d->cost,
-							do_count(mchxt, ctxt, w, rw, d->right, re, rnull_cnt));
-					}
-					if ((le == NULL) && (0 < hist_total(&rightcount)))
-					{
-						/* Evaluate using the right match, but not the left */
-						hist_muladdv(&total, &rightcount, d->cost,
-							do_count(mchxt, ctxt, lw, w, le, d->left, lnull_cnt));
+						CACHE_COUNT(l_bnr, hist_muladdv(&total, &leftcount, d->cost, count),
+							do_count(ctxt, w, rw, d->right, re, rnull_cnt));
 					}
+				}
 
-					/* Sigh. Overflows can and do occur, esp for the ANY language. */
-					if (INT_MAX < hist_total(&total))
+				if (rightpcount &&
+				    ((0 < hist_total(&leftcount)) || (0 != hist_total(&r_bnl))))
+				{
+					CACHE_COUNT(r_any, rightcount = count,
+						do_count(ctxt, w, rw, d->right->next, re->next, rnull_cnt));
+					if (re->multi)
+						CACHE_COUNT(r_cmulti, hist_accumv(&rightcount, d->cost, count),
+							do_count(ctxt, w, rw, d->right->next, re, rnull_cnt));
+					if (d->right->multi)
+						CACHE_COUNT(r_dmulti, hist_accumv(&rightcount, d->cost, count),
+							do_count(ctxt, w, rw, d->right, re->next, rnull_cnt));
+					if (d->right->multi && re->multi)
+						CACHE_COUNT(r_dcmulti, hist_accumv(&rightcount, d->cost, count),
+							do_count(ctxt, w, rw, d->right, re, rnull_cnt));
+
+					if (0 < hist_total(&rightcount))
 					{
-#ifdef PERFORM_COUNT_HISTOGRAMMING
-						total.total = INT_MAX;
-#else
-						total = INT_MAX;
-#endif /* PERFORM_COUNT_HISTOGRAMMING */
-						t->count = total;
-						pop_match_list(mchxt, mlb);
-						return total;
+						/* Total number where links are used on both sides */
+						hist_muladd(&total, &leftcount, 0.0, &rightcount);
+
+						/* Evaluate using the right match, but not the left */
+						if (le == NULL)
+							CACHE_COUNT(r_bnl, hist_muladdv(&total, &rightcount, d->cost, count),
+								do_count(ctxt, lw, w, le, d->left, lnull_cnt));
 					}
 				}
+
+				/* Sigh. Overflows can and do occur, esp for the ANY language. */
+				if (INT_MAX < hist_total(&total))
+				{
+#ifdef PERFORM_COUNT_HISTOGRAMMING
+					total.total = INT_MAX;
+#else
+					total = INT_MAX;
+#endif /* PERFORM_COUNT_HISTOGRAMMING */
+					t->count = total;
+					pop_match_list(mchxt, mlb);
+					return total;
+				}
 			}
 		}
 		pop_match_list(mchxt, mlb);
@@ -539,8 +598,9 @@ Count_bin do_parse(Sentence sent,
 	 * one null link. */
 	/* ctxt->null_block = 1; */
 	ctxt->islands_ok = opts->islands_ok;
+	ctxt->mchxt = mchxt;
 
-	hist = do_count(mchxt, ctxt, -1, sent->length, NULL, NULL, null_count+1);
+	hist = do_count(ctxt, -1, sent->length, NULL, NULL, null_count+1);
 
 	ctxt->local_sent = NULL;
 	ctxt->current_resources = NULL;
