@@ -181,29 +181,6 @@ struct prune_context_s
    deletable, this is equivalent to RUTHLESS.   --DS, 7/97
 */
 
-/**
- * returns the number of connectors in the left lists of the disjuncts.
- */
-static int left_connector_count(Disjunct * d)
-{
-	Connector *c;
-	int i=0;
-	for (;d!=NULL; d=d->next) {
-		for (c = d->left; c!=NULL; c = c->next) i++;
-	}
-	return i;
-}
-
-static int right_connector_count(Disjunct * d)
-{
-	Connector *c;
-	int i=0;
-	for (;d!=NULL; d=d->next) {
-	  for (c = d->right; c!=NULL; c = c->next) i++;
-	}
-	return i;
-}
-
 static void free_C_list(C_list * t)
 {
 	C_list *xt;
@@ -688,10 +665,8 @@ int power_prune(Sentence sent, Parse_Options opts)
 			}
 			sent->word[w].d = nd;
 		}
-		if (verbosity_level(D_PRUNE))
-		{
-			printf("l->r pass changed %d and deleted %zu\n", pc->N_changed, N_deleted);
-		}
+		lgdebug(D_PRUNE, "Debug: l->r pass changed %d and deleted %zu\n",
+		        pc->N_changed, N_deleted);
 
 		if (pc->N_changed == 0) break;
 
@@ -726,11 +701,8 @@ int power_prune(Sentence sent, Parse_Options opts)
 			sent->word[w].d = nd;
 		}
 
-		if (verbosity_level(D_PRUNE))
-		{
-			printf("r->l pass changed %d and deleted %zu\n",
-				pc->N_changed, N_deleted);
-		}
+		lgdebug(D_PRUNE, "Debug: r->l pass changed %d and deleted %zu\n",
+		        pc->N_changed, N_deleted);
 
 		if (pc->N_changed == 0) break;
 		pc->N_changed = N_deleted = 0;
@@ -740,13 +712,13 @@ int power_prune(Sentence sent, Parse_Options opts)
 	pt = NULL;
 	pc->pt = NULL;
 
-	if (verbosity_level(D_PRUNE))
-		printf("power prune cost: %d\n", pc->power_cost);
+	lgdebug(D_PRUNE, "Debug: power prune cost: %d\n", pc->power_cost);
 
 	print_time(opts, "power pruned");
 	if (verbosity_level(D_PRUNE))
 	{
-		printf("\nAfter power_pruning:\n");
+		prt_error("\n\\");
+		prt_error("Debug: After power_pruning:\n\\");
 		print_disjunct_counts(sent);
 	}
 
@@ -1108,15 +1080,15 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 			}
 		}
 
-		if (verbosity_level(D_PRUNE))
-			printf("pp_prune pass deleted %d\n", N_deleted);
+		lgdebug(D_PRUNE, "Debug: pp_prune pass deleted %d\n", N_deleted);
 	}
 	delete_unmarked_disjuncts(sent);
 	cms_table_delete(cmt);
 
-	if (verbosity_level(D_PRUNE))
+	if ((0 != N_deleted) && verbosity_level(D_PRUNE))
 	{
-		printf("\nAfter pp_pruning:\n");
+		prt_error("\n\\");
+		prt_error("Debug: After pp_pruning:\n\\");
 		print_disjunct_counts(sent);
 	}
 
