@@ -11,6 +11,7 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <limits.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <errno.h>
@@ -46,7 +47,7 @@ size_t utf8_strwidth(const char *s)
 	wchar_t *ws = alloca((mblen + 1) * sizeof(wchar_t));
 
 #ifdef _WIN32
-	MultiByteToWideChar(CP_UTF8, 0, s, -1, ws, mblen) - 1;
+	MultiByteToWideChar(CP_UTF8, 0, s, -1, ws, mblen);
 #else
 	mbstate_t mbss;
 	memset(&mbss, 0, sizeof(mbss));
@@ -61,6 +62,62 @@ size_t utf8_strwidth(const char *s)
 	return glyph_width;
 }
 
+/**
+ * Return the width, in text-column-widths, of the utf8-encoded
+ * character.
+ *
+ * The mbstate_t argument is not used, since we convert only from utf-8.
+ * FIXME: This function (along with other places that use mbrtowc()) need
+ * to be fixed for Windows (utf-16 wchar_t).
+ * Use char32_t with mbrtoc32() instead of mbrtowc().
+ */
+size_t utf8_charwidth(const char *s)
+{
+	wchar_t wc;
+
+	int n = mbrtowc(&wc, s, MB_LEN_MAX, NULL);
+	if (n == 0) return 0;
+	if (n < 0)
+	{
+		prt_error("Error: charwidth(%s): mbrtowc() returned %d\n", s, n);
+		return 1 /* XXX */;
+	}
+
+	return mk_wcwidth(wc);
+}
+
+/**
+ * Return the number of characters in the longest initial substring
+ * which has a text-column-width of not greater than max_width.
+ */
+size_t utf8_num_char(const char *s, size_t max_width)
+{
+	size_t total_bytes = 0;
+	size_t glyph_width = 0;
+	int n = 0;
+	wchar_t wc;
+
+	do
+	{
+		total_bytes += n;
+		n = mbrtowc(&wc, s+total_bytes, MB_LEN_MAX, NULL);
+		if (n == 0) break;
+		if (n < 0)
+		{
+			prt_error("Error: utf8_num_char(%s, %zu): mbrtowc() returned %d\n",
+			          s, max_width, n);
+			return 1 /* XXX */;
+		}
+
+		glyph_width += mk_wcwidth(wc);
+		//printf("N %zu G %zu;", total_bytes, glyph_width);
+	}
+	while (glyph_width <= max_width);
+	printf("\n");
+
+	return total_bytes;
+}
+
 /* ============================================================= */
 
 /**
@@ -75,14 +132,14 @@ int vappend_string(dyn_str * string, const char *fmt, va_list args)
 #define TMPLEN 1024 /* Big enough for a possible error message, see below */
 	char temp_buffer[TMPLEN];
 	char *temp_string = temp_buffer;
-	size_t templen;
+	int templen;
 	va_list copy_args;
 
 	va_copy(copy_args, args);
 	templen = vsnprintf(temp_string, TMPLEN, fmt, copy_args);
 	va_end(copy_args);
 
-	if ((int)templen < 0) goto error;
+	if (templen < 0) goto error;
 	if (0)
 	{
 		if (fmt[0] == '(') { errno=2; goto error;} /* Test the error reporting. */
@@ -94,7 +151,7 @@ int vappend_string(dyn_str * string, const char *fmt, va_list args)
 		 * find any example of entering this code with templen>=1024... */
 		temp_string = alloca(templen+1);
 		templen = vsnprintf(temp_string, templen+1, fmt, args);
-		if ((int)templen < 0) goto error;
+		if (templen < 0) goto error;
 	}
 	va_end(args);
 
@@ -130,7 +187,7 @@ size_t append_utf8_char(dyn_str * string, const char * mbs)
 {
 	/* Copy exactly one multi-byte character to buf */
 	char buf[10];
-	size_t n = utf8_next(mbs);
+	size_t n = utf8_charlen(mbs);
 
 	assert(n<10, "Multi-byte character is too long!");
 	strncpy(buf, mbs, n);
