@@ -116,8 +116,8 @@ static void left_append_string(dyn_str * string, const char * s, const char * t)
 		else
 			append_utf8_char(string, t);
 
-		s += utf8_next(s);
-		t += utf8_next(t);
+		s += utf8_charlen(s);
+		t += utf8_charlen(t);
 	}
 }
 
@@ -266,6 +266,11 @@ char * linkage_print_disjuncts(const Linkage linkage)
 		/* Subscript mark will be cleaned up by append_string(). */
 		infword = disj->word_string;
 
+		/* ... except that subscript marks confuse utf8_strwidth() */
+		/* linkage->word[w] should be same as disj->word_string except
+		 * that the subscript mark has been replaced by a dot. */
+		infword = linkage->word[w];
+
 		/* Make sure the glyphs align during printing. */
 		pad += strlen(infword) - utf8_strwidth(infword);
 
@@ -284,7 +289,18 @@ char * linkage_print_disjuncts(const Linkage linkage)
 }
 
 /**
- * postscript printing ...
+ * Postscript printing ...
+ * FIXME:
+ * 1. It is invoked after a call to linkage_print_diagram_ctxt() with a
+ *    screen width of 8000. But it actually cannot handle a screen width
+ *    greater than a page-width since it doesn't know to fold without the
+ *    help of the row_starts array which tells it on which word each
+ *    folded line starts (a garbled printout results).
+ * 2. It cannot handle utf-8 (garbage is printed).
+ * 3. Due to the added ability of folding long words, the row_starts
+ *    array is not sufficient for telling where to start the next line
+ *    (but this doesn't matter for now and can be fixed along with
+ *    problem no. 1 above).
  */
 static char *
 build_linkage_postscript_string(const Linkage linkage,
@@ -307,7 +323,7 @@ build_linkage_postscript_string(const Linkage linkage,
 			if (ppla[j].lw == 0) {
 				if (ppla[j].rw == linkage->num_words-1) continue;
 				N_wall_connectors ++;
-				if (strcmp(ppla[j].lc->string, LEFT_WALL_SUPPRESS)==0) {
+				if (easy_match(ppla[j].lc->string, LEFT_WALL_SUPPRESS)) {
 					suppressor_used = true;
 				}
 			}
@@ -323,7 +339,7 @@ build_linkage_postscript_string(const Linkage linkage,
 		for (j=0; j<N_links; j++) {
 			if (ppla[j].rw == linkage->num_words-1) {
 				N_wall_connectors ++;
-				if (strcmp(ppla[j].lc->string, RIGHT_WALL_SUPPRESS)==0){
+				if (easy_match(ppla[j].lc->string, RIGHT_WALL_SUPPRESS)) {
 					suppressor_used = true;
 				}
 			}
@@ -431,6 +447,13 @@ linkage_print_diagram_ctxt(const Linkage linkage,
                            size_t x_screen_width,
                            ps_ctxt_t *pctx)
 {
+	if (x_screen_width <= 2)
+	{
+		prt_error("Error: Cannot print diagram: "
+		          "Screen width %zu is too narrow.\n", x_screen_width);
+		return NULL;
+	}
+
 	bool display_short = true;
 	unsigned int i, j, k, cl, cr, inc, row, top_row, top_row_p1;
 	const char *s;
@@ -460,7 +483,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 			{
 				if (ppla[j].rw == linkage->num_words-1) continue;
 				N_wall_connectors ++;
-				if (0 == strcmp(ppla[j].lc->string, LEFT_WALL_SUPPRESS))
+				if (easy_match(ppla[j].lc->string, LEFT_WALL_SUPPRESS))
 				{
 					suppressor_used = true;
 				}
@@ -481,7 +504,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 			if (ppla[j].rw == linkage->num_words-1)
 			{
 				N_wall_connectors ++;
-				if (0 == strcmp(ppla[j].lc->string, RIGHT_WALL_SUPPRESS))
+				if (easy_match(ppla[j].lc->string, RIGHT_WALL_SUPPRESS))
 				{
 					suppressor_used = true;
 				}
@@ -534,8 +557,11 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 				if (k == cr) break;
 			}
 
-			/* We know it fits, so put it in this row */
-			pctx->link_heights[j] = row;
+			if (NULL != pctx) /* PS junk */
+			{
+				/* We know it fits, so put it in this row */
+				pctx->link_heights[j] = row;
+			}
 
 			if (2*row+2 > max_height-1) {
 				lgdebug(+9, "Extending rows up to %d.\n", (2*row+2)+HEIGHT_INC);
@@ -645,27 +671,44 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	top_row_p1 = top_row + 1;
 	for (row = 0; row < top_row_p1; row++)
 		start[row] = 0;
-	pctx->N_rows = 0;
-	pctx->row_starts[pctx->N_rows] = 0;
-	pctx->N_rows++;
+
+	if (NULL != pctx) /* PS junk */
+	{
+		pctx->N_rows = 0;
+		pctx->row_starts[pctx->N_rows] = 0;
+		pctx->N_rows++;
+	}
 
 	if (print_word_0) i = 0; else i = 1;
+	unsigned int c = 0; /* Character offset in the last word on a row. */
+#define RIGHT_MARGIN 1
 	while (i < N_words_to_print)
 	{
 		unsigned int revrs;
-		/* Count the column-widths of the words,
-		 * up to the max screen width. */
+		/* Count the column-widths of the words, up to the max screen width.
+		 * Use word_offset only for the initial part of the word. */
 		unsigned int uwidth = 0;
 		unsigned int wwid;
 		do {
-			wwid = word_offset[i] + utf8_strwidth(linkage->word[i]) + 1;
-			if (x_screen_width <= uwidth + wwid) break;
+			wwid = (c == 0)*word_offset[i] + utf8_strwidth(linkage->word[i]+c) + 1;
+			if (x_screen_width-RIGHT_MARGIN < uwidth + wwid) break;
 			uwidth += wwid;
+			c = 0;
 			i++;
 		} while (i < N_words_to_print);
 
-		pctx->row_starts[pctx->N_rows] = i - (!print_word_0);    /* PS junk */
-		if (i < N_words_to_print) pctx->N_rows++;     /* same */
+		/* The whole word doesn't fit - fit as much as possible from it. */
+		if (0 == uwidth)
+		{
+			uwidth = x_screen_width-RIGHT_MARGIN - (c == 0)*word_offset[i] - 1;
+			c += utf8_num_char(linkage->word[i]+c, uwidth);
+		}
+
+		if (NULL != pctx) /* PS junk */
+		{
+			pctx->row_starts[pctx->N_rows] = i - (!print_word_0);
+			if (i < N_words_to_print) pctx->N_rows++;
+		}
 
 		dyn_strcat(string, "\n");
 		top_row_p1 = top_row + 1;
@@ -673,42 +716,36 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 		{
 			/* print each row of the picture */
 			/* `blank` is used solely to detect blank lines */
-			bool blank = true;
+			unsigned int last_nonblank = (unsigned int)-1;
 			/*
 			 * The `glyph_width` is the width, in columns, of the printable
 			 * glyph. Chinese glyphs are almost always width two.
 			 */
-			unsigned int glyph_width = 0;
-
-			wchar_t  wc;
-			mbstate_t mbss;
-			memset(&mbss, 0, sizeof(mbss));
+			size_t glyph_width = 0;
 
 			row = top_row - revrs;
 			k = start[row];
 			for (j = k; (glyph_width < uwidth) && (xpicture[row][j] != '\0'); )
 			{
-				blank = blank && (xpicture[row][j] == ' ');
+				if (xpicture[row][j] != ' ') last_nonblank = j;
 
 				/* Update the printable column width */
-				mbrtowc(&wc, &xpicture[row][j], MB_CUR_MAX, &mbss);
-				glyph_width += mk_wcwidth(wc);
+				glyph_width += utf8_charwidth(&xpicture[row][j]);
 
-				j += utf8_next(&xpicture[row][j]);
+				j += utf8_charlen(&xpicture[row][j]);
 			}
 			start[row] = j;
 
-			if (!blank)
+			if ((unsigned int)-1 != last_nonblank)
 			{
-				memset(&mbss, 0, sizeof(mbss));
 				glyph_width = 0;
 				for (j = k; (glyph_width < uwidth) && (xpicture[row][j] != '\0'); )
 				{
-					mbrtowc(&wc, &xpicture[row][j], MB_CUR_MAX, &mbss);
-					glyph_width += mk_wcwidth(wc);
+					glyph_width += utf8_charwidth(&xpicture[row][j]);
 
 					/* Copy exactly one multi-byte character to buf */
 					j += append_utf8_char(string, &xpicture[row][j]);
+					if (last_nonblank < j) break; /* Trim trailing blanks */
 				}
 				dyn_strcat(string, "\n");
 			}
@@ -730,16 +767,14 @@ linkage_print_diagram_ctxt(const Linkage linkage,
  */
 char * linkage_print_diagram(const Linkage linkage, bool display_walls, size_t screen_width)
 {
-	ps_ctxt_t ctx;
 	if (!linkage) return NULL;
 
-	ctx.link_heights = (int *) alloca(linkage->num_links * sizeof(int));
-	ctx.row_starts = (int *) alloca((linkage->num_words + 1) * sizeof(int));
-	return linkage_print_diagram_ctxt(linkage, display_walls, screen_width, &ctx);
+	return linkage_print_diagram_ctxt(linkage, display_walls, screen_width, NULL);
 }
 
 void linkage_free_diagram(char * s)
 {
+	if (NULL == s) return;
 	exfree(s, strlen(s)+1);
 }
 
@@ -804,17 +839,25 @@ void linkage_free_pp_msgs(char * s)
 void print_disjunct_counts(Sentence sent)
 {
 	size_t i;
-	int c;
-	Disjunct *d;
+	int dcnt;
+	int t = 0;
+	int lcnt = 0, rcnt = 0;
+
 	for (i=0; i<sent->length; i++)
 	{
-		c = 0;
-		for (d=sent->word[i].d; d != NULL; d = d->next) c++;
+		Disjunct *d = sent->word[i].d;
+		dcnt = count_disjuncts(d);
+		rcnt += right_connector_count(d);
+		lcnt += left_connector_count(d);
+		t += dcnt;
 
 		/* XXX alternatives[0] is not really correct, here .. */
-		printf("%s(%d) ", sent->word[i].alternatives[0], c);
+		prt_error("%s(%d) ", sent->word[i].alternatives[0], dcnt);
 	}
-	printf("\n\n");
+
+	prt_error("\n\\");
+	prt_error("Total: %d disjuncts, %d (%d+/%d-) connectors\n\n",
+	          t, rcnt+lcnt, rcnt, lcnt);
 }
 
 static const char * trailer(bool print_ps_header)
@@ -1192,20 +1235,20 @@ static const char * header(bool print_ps_header)
 /**
  * Print elements of the 2D-word-array produced for the parsers.
  *
- * - print_sentence_word_alternatives(sent, false, NULL, tokenpos)
+ * - print_sentence_word_alternatives(s, sent, false, NULL, tokenpos)
  * If a pointer to struct "tokenpos" is given, return through it the index of
  * the first occurrence in the sentence of the given token. This is used to
  * prevent duplicate information display for repeated morphemes (if there are
  * multiples splits, each of several morphemes, otherwise some of them may
  * repeat).
  *
- * - print_sentence_word_alternatives(sent, true, NULL, NULL)
+ * - print_sentence_word_alternatives(s, sent, true, NULL, NULL)
  * If debugprint is "true", this is a debug printout of the sentence.  (The
  * debug printouts are with level 0 because this function is invoked for debug
  * on certain positive level.)
  *
  *
- * - print_sentence_word_alternatives(sent, false, display_func, NULL)
+ * - print_sentence_word_alternatives(s, sent, false, display_func, NULL)
  * Iterate over the sentence words and their alternatives.  Handle each
  * alternative using the display_func function if it is supplied, or else (if it
  * is NULL) just print them. It is used to display disjunct information when
@@ -1333,17 +1376,25 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 		{
 			if (debugprint)
 			{
-				if (0 < ai) dyn_strcat(s, "\n   ");
+				if (0 < ai) dyn_strcat(s, "\n");
 				append_string(s, "   alt%zu:", ai);
 			}
 
+			int missing = 0;
 			for (wi = w_start; (wi == w_start) ||
 			    ((wi < sentlen) && (! sent->word[wi].unsplit_word)); wi++)
 			{
 				size_t nalts = altlen(sent->word[wi].alternatives);
 				const char *wt;
 
-				if (ai >= nalts) continue;
+				/* Some slots may be shorter then others. To avoid a
+				 * misleading debug display, print "[missing]" for them,
+				 * but only if there are tokens after them. */
+				if (ai >= nalts)
+				{
+					missing++;
+					continue;
+				}
 				wt = sent->word[wi].alternatives[ai];
 
 				/* Don't display information again for the same word */
@@ -1370,38 +1421,39 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 
 				if (debugprint)
 				{
+					while (missing-- > 0) append_string(s, " %s", "[missing]");
+
 					const char *opt_start = "", *opt_end = "";
 					if (sent->word[wi].optional)
 					{
 						opt_start = "{";
 						opt_end = "}";
 					}
+					if ('\0' == wt[0]) wt = "'\\0'"; /* Reveal a bogus value. */
 					append_string(s, " %s%s%s", opt_start, wt, opt_end);
 				}
 
-				/* Don't try to give info on the empty word. */
-				if ('\0' != wt[0])
+				/* For now each word component is called "Token".
+				 * TODO: Its type can be decoded and a more precise
+				 * term (stem, prefix, etc.) can be used.
+				 * Display the features of the token. */
+				if ((NULL == tokenpos) && (NULL != display))
 				{
-					/* For now each word component is called "Token".
-					 * TODO: Its type can be decoded and a more precise
-					 * term (stem, prefix, etc.) can be used.
-					 * Display the features of the token */
-					if ((NULL == tokenpos) && (NULL != display))
-					{
-						char *info = display(sent->dict, wt);
+					char *info = display(sent->dict, wt);
 
-						if (NULL == info) return;
-						append_string(s, "Token \"%s\" %s", wt, info);
-						free(info);
-					}
-					else if (word_split) append_string(s, " %s", wt);
+					if (NULL == info) return;
+					append_string(s, "Token \"%s\" %s", wt, info);
+					free(info);
 				}
+				else if (word_split) append_string(s, " %s", wt);
 			}
 
-			/* Commented out - no alternatives for now - print as one line. */
-			//if (word_split && (NULL == display)) dyn_strcat(s, "\n");
 		}
 		wi--;
+
+		/* Line separation after the tokens printed by "String splits to:". */
+		if (word_split && (NULL == display)) dyn_strcat(s, "\n\n");
+
 		if (debugprint) dyn_strcat(s, "\n");
 	}
 	if (debugprint) dyn_strcat(s, "\n");
