@@ -1,8 +1,10 @@
 #include "util.hpp"
 
 extern "C" {
+#include "api-structures.h"
+#include "disjunct-utils.h"
+#include "linkage/linkage.h"
 #include "utilities.h"
-#include "linkage.h"
 };
 
 /**
