@@ -6,7 +6,8 @@
 #include <set>
 
 extern "C" {
-#include "word-utils.h"
+#include "connectors.h"
+#include "dict-common/dict-common.h"
 };
 
 #include "variables.hpp"
