@@ -13,6 +13,9 @@
 
 #include <ctype.h>
 #include <limits.h>
+#ifdef _WIN32
+#define _CRT_RAND_S
+#endif /* _WIN32 */
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -88,9 +91,7 @@ char *
 strndup (const char *str, size_t size)
 {
 	size_t len;
-	char *result = (char *) NULL;
-
-	if ((char *) NULL == str) return (char *) NULL;
+	char *result;
 
 	len = strlen (str);
 	if (!len) return strdup ("");
@@ -131,25 +132,6 @@ char* strtok_r(char *str, const char *delim, char **nextp)
 /* ============================================================= */
 /* UTF8 utilities */
 
-/** Returns length of UTF8 character.
- * Current algo is based on the first character only.
- * If pointer is not pointing at first char, no not a valid value, returns 0.
- * Returns 0 for NULL as well.
- */
-int utf8_charlen(const char *xc)
-{
-	unsigned char c;
-
-	c = (unsigned char) *xc;
-
-	if (c == 0) return 0;
-	if (c < 0x80) return 1;
-	if ((c >= 0xc2) && (c < 0xe0)) return 2; /* First byte of a code point U +0080 - U +07FF */
-	if ((c >= 0xe0) && (c < 0xf0)) return 3; /* First byte of a code point U +0800 - U +FFFF */
-	if ((c >= 0xf0) && (c <= 0xf4)) return 4; /* First byte of a code point U +10000 - U +10FFFF */
-	return -1; /* Fallthrough -- not the first byte of a code-point. */
-}
-
 #ifdef _WIN32
 /**
  * (Experimental) Implementation of mbrtowc for Windows.
@@ -173,6 +155,22 @@ size_t lg_mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
 	if (0 == nb2) return (size_t)-1;
 	return nb;
 }
+
+/**
+ * Emulate rand_r() using rand_s() in a way that is enough for our needs.
+ * Windows doesn't have rand_r(), and its rand_s() is different: It
+ * returns an error indication and not the random number like rand_r().
+ * The value its returns is through its argument.
+ *
+ * Note that "#define _CRT_RAND_S" is needed before "#include <stdlib.h>"
+ */
+int rand_r(unsigned int *s)
+{
+	rand_s(s);
+	if (*s > INT_MAX) *s -= INT_MAX;
+
+	return *s;
+}
 #endif /* _WIN32 */
 
 static int wctomb_check(char *s, wchar_t wc)
@@ -201,7 +199,7 @@ static int wctomb_check(char *s, wchar_t wc)
  * because the byte-counts might not match up, e.g. German ß and SS.
  * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
-void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale_t)
+void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale)
 {
 	wchar_t c;
 	int i, nbl, nbh;
@@ -218,7 +216,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t local
 		prt_error("Error: Invalid UTF-8 string!\n");
 		return;
 	}
-	c = towlower_l(c, locale_t);
+	c = towlower_l(c, locale);
 	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
@@ -247,7 +245,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t local
  * because the byte-counts might not match up, e.g. German ß and SS.
  * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
-void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale_t)
+void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale)
 {
 	wchar_t c;
 	int i, nbl, nbh;
@@ -261,7 +259,7 @@ void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale_
 		prt_error("Error: Invalid UTF-8 string!\n");
 		return;
 	}
-	c = towupper_l(c, locale_t);
+	c = towupper_l(c, locale);
 	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
