@@ -19,9 +19,6 @@
 
 #include <ctype.h>
 #include <stdio.h>
-#ifdef _WIN32
-#define _CRT_RAND_S
-#endif /* _WIN32 */
 #include <stdlib.h>
 #include <string.h>
 #include <wchar.h>
@@ -61,11 +58,6 @@ void *alloca (size_t);
 #endif /* _MSC_VER */
 #endif /* !TLS */
 
-#ifndef strdupa
-/* In the following, the argument should not have side effects. */
-#define strdupa(s) strcpy(alloca(strlen(s)+1), s)
-#endif
-
 /* Windows, POSIX and GNU have different ideas about thread-safe strerror(). */
 #ifdef _WIN32
 #define strerror_r(errno, buf, len) strerror_s(buf, len, errno)
@@ -109,19 +101,22 @@ void *alloca (size_t);
 #ifndef strncasecmp
 #define strncasecmp(a,b,s) strnicmp((a),(b),(s))
 #endif
-/* Note that "#define _CRT_RAND_S" is needed before "#include <stdlib.h>" */
-#define rand_r(seedp) rand_s(seedp)
-
-/* No strtok_s in these versions and their strtok_r is incompatible. */
+int rand_r(unsigned int *);
+#ifndef __MINGW32__
+/* No strtok_s in XP/2003 and their strtok_r is incompatible.
+ * Hence HAVE_STRTOK_R will not be defined and our own one will be used. */
 #if _WINVER != 0x501 /* XP */ && _WINVER != 0x502 /* Server 2003 */
 #define strtok_r strtok_s
 #define HAVE_STRTOK_R
-#endif
+#endif /* _WINVER != XP|2003 */
+
+/* There is no ssize_t definition in native Windows. */
+#include <BaseTsd.h>
+typedef SSIZE_T ssize_t;
 
 /* Native windows has locale_t, and hence HAVE_LOCALE_T is defined here.
  * However, MinGW currently doesn't have locale_t. If/when it has locale_t,
  * "configure" will define HAVE_LOCALE_T for it. */
-#ifndef __MINGW32__
 #define HAVE_LOCALE_T
 #endif
 
@@ -151,41 +146,10 @@ size_t lg_mbrtowc(wchar_t *, const char *, size_t n, mbstate_t *ps);
 #endif /* _WIN32 */
 
 /* MSVC isspace asserts in debug mode, and mingw sometime returns true,
- * when passed utf8. Thus, limit to 7 bits for windows. */
-#ifdef _WIN32
-  #define lg_isspace(c) ((0 < c) && (c < 127) && isspace(c))
-#else
-  #define lg_isspace isspace
-#endif
-
-#if __APPLE__
-/* It appears that fgetc on Mac OS 10.11.3 "El Capitan" has a weird
- * or broken version of fgetc() that flubs reads of utf8 chars when
- * the locale is not set to "C" -- in particular, it fails for the
- * en_US.utf8 locale; see bug report #293
- * https://github.com/opencog/link-grammar/issues/293
- */
-static inline int lg_fgetc(FILE *stream)
-{
-	char c[4];  /* general overflow paranoia */
-	size_t nr = fread(c, 1, 1, stream);
-	if (0 == nr) return EOF;
-	return (int) c[0];
-}
-
-static inline int lg_ungetc(int c, FILE *stream)
-{
-	/* This should work, because we never unget past the newline char. */
-	int rc = fseek(stream, -1, SEEK_CUR);
-	if (rc) return EOF;
-	return c;
-}
-
-#else
-#define lg_fgetc   fgetc
-#define lg_ungetc  ungetc
-#endif
-
+ * when passed utf8. OSX returns TRUE on char values 0x85 and 0xa0).
+ * Since it is defined to return TRUE only on 6 characters, all of which
+ * are in the range [0..127], just limit its arguments to 7 bits. */
+#define lg_isspace(c) ((0 < c) && (c < 127) && isspace(c))
 
 #if defined(__sun__)
 int strncasecmp(const char *s1, const char *s2, size_t n);
@@ -221,6 +185,22 @@ typedef int locale_t;
 #define MAX(X,Y)  ( ((X) > (Y)) ? (X) : (Y))
 #endif
 
+/* In the following, the arguments should not have side effects.
+ * FIXME: Detect in "configure" and check HAVE_* */
+#ifndef strdupa
+#define strdupa(s) strcpy(alloca(strlen(s)+1), s)
+#endif
+#ifndef strndupa
+#define strndupa(s, n) _strndupa3(alloca((n)+1), s, n)
+static inline char *_strndupa3(char *new_s, const char *s, size_t n)
+{
+	strncpy(new_s, s, n);
+	new_s[n] = '\0';
+
+	return new_s;
+}
+#endif
+
 /* From ccan array_size.h and build_assert.h, which are under a CC0 license */
 #define BUILD_ASSERT_OR_ZERO(cond) (sizeof(char [1 - 2*!(cond)]) - 1)
 #if !defined(ARRAY_SIZE)
@@ -254,6 +234,37 @@ typedef int locale_t;
 #define GNUC_UNUSED
 #endif
 
+#if ((__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7)) || __clang__)
+#define GCC_DIAGNOSTIC
+#endif
+
+/* Apply a pragma to a specific code section only.
+ * XXX According to the GCC docs, we cannot use here something like
+ * "#ifdef HAVE_x". Also -Wunknown-pragmas & -Wno-unknown-warning-option
+ * don't work in this situation. So "-Wmaybe-uninitialized", which
+ * is not recognized by clang, is defined separately. */
+#ifdef GCC_DIAGNOSTIC
+
+#ifdef HAVE_MAYBE_UNINITIALIZED
+#define PRAGMA_MAYBE_UNINITIALIZED \
+	_Pragma("GCC diagnostic push") \
+	_Pragma("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
+#else
+#define PRAGMA_MAYBE_UNINITIALIZED \
+	_Pragma("GCC diagnostic push")
+#endif /* HAVE_MAYBE_UNINITIALIZED */
+
+#define PRAGMA_START(x) \
+	_Pragma("GCC diagnostic push") \
+	_Pragma("GCC diagnostic ignored \"-Wunknown-pragmas\"") \
+	_Pragma(#x)
+#define PRAGMA_END _Pragma("GCC diagnostic pop")
+#else
+#define PRAGMA_START(x)
+#define PRAGMA_END
+#define PRAGMA_MAYBE_UNINITIALIZED
+#endif /* GCC_DIAGNOSTIC */
+
 /**
  * Return the length, in codepoints/glyphs, of the utf8-encoded
  * string.  The string is assumed to be null-terminated.
@@ -263,55 +274,31 @@ static inline size_t utf8_strlen(const char *s)
 {
 	mbstate_t mbss;
 	memset(&mbss, 0, sizeof(mbss));
-#if defined(_MSC_VER) || defined(__MINGW32__)
+#if _WIN32
 	return MultiByteToWideChar(CP_UTF8, 0, s, -1, NULL, 0)-1;
 #else
 	return mbsrtowcs(NULL, &s, 0, &mbss);
-#endif
-}
-
-/**
- * Return the distance, in bytes, to the next character, in the
- * input utf8-encoded string.
- */
-static inline size_t utf8_next(const char *s)
-{
-#ifdef _WIN32
-	/* mbrlen does not work correctly on Windows. See issue #285 */
-	/* https://github.com/opencog/link-grammar/issues/285 */
-	size_t len = 0;
-	while (0 != *s)
-	{
-		if ((0x80 <= ((unsigned char) *s)) &&
-		(((unsigned char) *s) < 0xc0)) { s++; len++; }
-		else return len+1;
-	}
-	return len;
-#else
-	size_t len;
-	mbstate_t mbs;
-	memset(&mbs, 0, sizeof(mbs));
-	len = mbrlen(s, MB_CUR_MAX, &mbs);
-	if (len == (size_t)(-1) || len == (size_t)(-2)) {
-		/* Too long or malformed sequence, step one byte. */
-		return 1;
-	}
-	return len;
 #endif /* _WIN32 */
 }
 
-/**
- * Return the length, in codepoints/glyphs, of the utf8-encoded
- * string.  The string is assumed to be at least `len` code-points
- * long. This is needed when splitting words into morphemes.
+/** Returns length of UTF8 character.
+ * Current algo is based on the first character only.
+ * If pointer is not pointing at first char, or not a valid value, returns -1.
+ * Returns 0 for NULL.
  */
-static inline size_t utf8_strnlen(const char *s, size_t len)
+static inline int utf8_charlen(const char *xc)
 {
-	size_t by = 0;
-	while (0 < len) { by += utf8_next(&s[by]); }
-	return by;
-}
+	unsigned char c;
 
+	c = (unsigned char) *xc;
+
+	if (c == 0) return 0;
+	if (c < 0x80) return 1;
+	if ((c >= 0xc2) && (c < 0xe0)) return 2; /* First byte of a code point U +0080 - U +07FF */
+	if ((c >= 0xe0) && (c < 0xf0)) return 3; /* First byte of a code point U +0800 - U +FFFF */
+	if ((c >= 0xf0) && (c <= 0xf4)) return 4; /* First byte of a code point U +10000 - U +10FFFF */
+	return -1; /* Fallthrough -- not the first byte of a code-point. */
+}
 
 /**
  * Copy `n` utf8 characters from `src` to `dest`.
@@ -323,7 +310,7 @@ static inline size_t utf8_strncpy(char *dest, const char *src, size_t n)
 	size_t b = 0;
 	while (0 < n)
 	{
-		size_t k = utf8_next(src);
+		size_t k = utf8_charlen(src);
 		b += k;
 		while (0 < k) { *dest = *src; dest++; src++; k--; }
 		n--;
@@ -437,7 +424,6 @@ void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t);
 #if 0
 void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t);
 #endif
-int utf8_charlen(const char *);
 
 size_t lg_strlcpy(char * dest, const char *src, size_t size);
 void safe_strcpy(char *u, const char * v, size_t usize);
