@@ -135,7 +135,12 @@ static void clean_up_string(char * s)
 	while(*x != '\0')
 	{
 		w = mbrtowc(&p, x, len, &state);
-		if ((0 == w) || ((size_t)-1 == w)) break;
+		if (0 == w) break;
+		if (0 > (ssize_t)w)
+		{
+			prt_error("Unable to process UTF8 command input string.\n");
+			break;
+		}
 		len -= w;
 
 		if (!iswspace(p)) {
@@ -179,7 +184,11 @@ static bool is_numerical_rhs(char *s)
 	{
 		w = mbrtowc(&p, s, len, &state);
 		if (0 == w) break;
-		if ((size_t)-1 == w) return false;
+		if (0 > (ssize_t)w)
+		{
+			prt_error("Unable to process UTF8 command input string.\n");
+			break;
+		}
 		len -= w;
 		if (!iswdigit(p)) return false;
 	}
@@ -549,7 +558,7 @@ Command_Options* command_options_create(void)
 	co->panic_opts = parse_options_create();
 
 	/* "Unlimited" screen width when writing to a file, auto-updated
-	 * later, wen writing to a tty. */
+	 * later, when writing to a tty. */
 	co->screen_width = 16381;
 	co->allow_null = true;
 	co->batch_mode = false;
