@@ -37,6 +37,7 @@
 #include <locale.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 
 /* Used for terminal resizing */
 #ifndef _WIN32
@@ -128,7 +129,7 @@ static char * fget_input_string(FILE *in, FILE *out, bool check_return)
 
 	input_string[MAX_INPUT-2] = '\0';
 
-	if ((in != stdin) || !isatty_stdin)
+	if (((in != stdin) && !check_return) || !isatty_stdin)
 	{
 		/* Get input from a file. */
 		pline = fgets(input_string, MAX_INPUT, in);
@@ -139,7 +140,7 @@ static char * fget_input_string(FILE *in, FILE *out, bool check_return)
 		pline = get_terminal_line(input_string, in, out);
 	}
 
-	if (NULL == pline) return NULL;      /* EOF */
+	if (NULL == pline) return NULL;      /* EOF or error */
 
 	if (('\0' != input_string[MAX_INPUT-2]) &&
 	    ('\n' != input_string[MAX_INPUT-2]))
@@ -152,7 +153,8 @@ static char * fget_input_string(FILE *in, FILE *out, bool check_return)
 	{
 		if (('\0' == pline[0]) || ('\r' == pline[0]) || ('\n' == pline[0]))
 			return (char *)"\n";           /* Continue linkage display */
-		input_pending = true;
+		if ((in == stdin) || ('!' == pline[0]))
+			input_pending = true;          /* In !file mode allow commands */
 		return (char *)"x";               /* Stop linkage display */
 	}
 
@@ -380,11 +382,11 @@ static const char *process_some_linkages(FILE *in, Sentence sent,
 		{
 			if (!auto_next_linkage)
 			{
-				if ((verbosity > 0) && (in == stdin) && isatty_stdin && isatty_stdout)
+				if ((verbosity > 0) && (!copts->batch_mode) && isatty_stdin && isatty_stdout)
 				{
 					fprintf(stdout, "Press RETURN for the next linkage.\n");
 				}
-				char *rc = fget_input_string(in, stdout, /*check_return*/true);
+				char *rc = fget_input_string(stdin, stdout, /*check_return*/true);
 				if ((NULL == rc) || (*rc != '\n')) return rc;
 			}
 		}
@@ -507,7 +509,6 @@ static void print_usage(char *str)
 /**
  * On Unix, this checks for the current window size,
  * and sets the output screen width accordingly.
- * Not sure how MS Windows does this.
  */
 static void check_winsize(Command_Options* copts)
 {
@@ -524,11 +525,11 @@ static void check_winsize(Command_Options* copts)
 	/* Calculate the size of the console window. */
 	if (GetConsoleScreenBufferInfo(console, &info) == 0) goto fail;
 
-	copts->screen_width = info.srWindow.Right - info.srWindow.Left;
+	copts->screen_width = info.srWindow.Right - info.srWindow.Left + 1;
 	return;
 
 fail:
-	copts->screen_width = 79;
+	copts->screen_width = 80;
 	return;
 #else
 	struct winsize ws;
@@ -552,7 +553,7 @@ fail:
 	 */
 	if ((10 < ws.ws_col) && (16123 > ws.ws_col))
 	{
-		copts->screen_width = ws.ws_col - 1;
+		copts->screen_width = ws.ws_col;
 	}
 #endif /* _WIN32 */
 }
@@ -572,7 +573,7 @@ int main(int argc, char * argv[])
 	isatty_stdout = isatty(fileno(stdout));
 
 #ifdef _WIN32
-	/* If compiled with MSVC/MSYS, we still support running under Cygwin.
+	/* If compiled with MSVC/MinGW, we still support running under Cygwin.
 	 * This is done by checking running_under_cygwin to resolve
 	 * incompatibilities. */
 	const char *ostype = getenv("OSTYPE");
@@ -682,6 +683,10 @@ int main(int argc, char * argv[])
 		char *input_string;
 		Sentence sent = NULL;
 
+		/* Make sure stderr is shown even when MSVC binary runs under
+		 * Cygwin/MSYS pty (in that case it is fully buffered(!). */
+		fflush(stderr);
+
 		verbosity = parse_options_get_verbosity(opts);
 		debug = parse_options_get_debug(opts);
 		test = parse_options_get_test(opts);
@@ -691,6 +696,9 @@ int main(int argc, char * argv[])
 
 		if (NULL == input_string)
 		{
+			if (ferror(input_fh))
+				prt_error("Error: Read: %s\n", strerror(errno));
+
 			if (input_fh == stdin) break;
 			fclose (input_fh);
 			input_fh = stdin;
@@ -716,12 +724,20 @@ int main(int argc, char * argv[])
 
 			if ('\n' == filename[fnlen-1]) filename[fnlen-1] = '\0';
 
+			struct stat statbuf;
+			if ((0 == stat(filename, &statbuf)) && statbuf.st_mode & S_IFDIR)
+			{
+				fprintf(stderr, "Error: Cannot open %s: %s\n",
+				        filename, strerror(EISDIR));
+				continue;
+			}
+
 			input_fh = fopen(filename, "r");
+
 			if (NULL == input_fh)
 			{
-				int perr = errno;
-				fprintf(stderr, "Error: %s (%d) %s\n",
-				        filename, perr, strerror(perr));
+				fprintf(stderr, "Error: Cannot open %s: %s\n",
+				        filename, strerror(errno));
 				input_fh = stdin;
 				continue;
 			}
@@ -891,24 +907,21 @@ int main(int argc, char * argv[])
 
 			/* print_total_time(opts); */
 
+			const char *rc = "";
 			if (copts->batch_mode)
 			{
 				batch_process_some_linkages(label, sent, copts);
 			}
 			else
 			{
-				const char *rc = process_some_linkages(input_fh, sent, copts);
-				if (NULL == rc)
-				{
-					sentence_delete(sent);
-					sent = NULL;
-					break;
-				}
+				rc = process_some_linkages(input_fh, sent, copts);
 			}
-			fflush(stdout);
 
+			fflush(stdout);
 			sentence_delete(sent);
 			sent = NULL;
+
+			if ((NULL == rc) && (input_fh == stdin)) break;
 		}
 	}
 
